using View_by_Distance.Shared.Models.Stateless;

namespace View_by_Distance.Property.Models.Stateless;

internal class Result
{

    internal static string GetRelativePath(Models.Configuration configuration, string path)
    {
        string result = XPath.GetRelativePath(path, configuration.RootDirectory.Length);
        return result;
    }

    internal static string GetResultsGroupDirectory(Models.Configuration configuration, string description)
    {
        string result = Path.Combine($"{configuration.RootDirectory} - Results", description.Replace("_", ") "));
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsDateGroupDirectory(Models.Configuration configuration, string description)
    {
        string result = Path.Combine(GetResultsGroupDirectory(configuration, description), configuration.DateGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsDateGroupDirectory(Models.Configuration configuration, string description, string jsonGroup)
    {
        string result = Path.Combine(GetResultsDateGroupDirectory(configuration, description), jsonGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsFullGroupDirectory(Models.Configuration configuration, Model? model, PredictorModel? predictorModel, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel)
    {
        string result = GetResultsDateGroupDirectory(configuration, description);
        if (includeResizeGroup)
            result = Path.Combine(result, outputResolution);
        if (includeModel && includePredictorModel)
        {
            string modelName;
            string predictorModelName;
            if (model is null)
                modelName = Model.Hog.ToString();
            else
                modelName = model.Value.ToString();
            if (predictorModel is null)
                predictorModelName = PredictorModel.Large.ToString();
            else
                predictorModelName = predictorModel.Value.ToString();
            string dateGroupDirectory = string.Concat(outputResolution.Replace(" ", string.Empty), " - ", modelName, " - ", predictorModelName);
            result = Path.Combine(result, dateGroupDirectory);
        }
        else if (includeModel)
            throw new Exception();
        else if (includePredictorModel)
            throw new Exception();
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static List<string> GetDirectoryInfoCollection(Models.Configuration configuration, string sourceDirectory, string dateGroupDirectory, string contentDescription, string singletonDescription, string collectionDescription)
    {
        List<string> results = new();
        string result = string.Empty;
        string checkDirectory;
        const string allInOne = "_ _ _";
        string sourceDirectorySegment = GetRelativePath(configuration, sourceDirectory);
        if (!string.IsNullOrEmpty(contentDescription))
        {
            checkDirectory = Path.Combine(dateGroupDirectory, "()", allInOne);
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            string contentDirectory = new(result.Replace("<>", "()"));
            if (!Directory.Exists(contentDirectory))
                _ = Directory.CreateDirectory(contentDirectory);
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("() - ", contentDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (!string.IsNullOrEmpty(singletonDescription))
        {
            checkDirectory = Path.Combine(dateGroupDirectory, "{}", allInOne);
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            string singletonDirectory = new(result.Replace("<>", "{}"));
            if (!Directory.Exists(singletonDirectory))
                _ = Directory.CreateDirectory(singletonDirectory);
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("{} - ", singletonDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (!string.IsNullOrEmpty(collectionDescription))
        {
            checkDirectory = Path.Combine(dateGroupDirectory, "[]", allInOne);
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            string collectionDirectory = new(result.Replace("<>", "[]"));
            if (!Directory.Exists(collectionDirectory))
                _ = Directory.CreateDirectory(collectionDirectory);
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("[] - ", collectionDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (string.IsNullOrEmpty(result))
            throw new Exception();
        results.Add(result);
        return results;
    }

    internal static List<string> GetDirectoryInfoCollection(Models.Configuration configuration, Model? model, PredictorModel? predictorModel, string sourceDirectory, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel, string contentDescription, string singletonDescription, string collectionDescription)
    {
        List<string> results = new();
        string result = string.Empty;
        string checkDirectory;
        string sourceDirectorySegment = GetRelativePath(configuration, sourceDirectory);
        string dateGroupDirectory = GetResultsFullGroupDirectory(configuration, model, predictorModel, description, outputResolution, includeResizeGroup, includeModel, includePredictorModel);
        if (!string.IsNullOrEmpty(contentDescription))
        {
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            System.IO.DirectoryInfo contentDirectoryInfo = new(result.Replace("<>", "()"));
            if (!contentDirectoryInfo.Exists)
                contentDirectoryInfo.Create();
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("() - ", contentDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (!string.IsNullOrEmpty(singletonDescription))
        {
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            System.IO.DirectoryInfo singletonDirectoryInfo = new(result.Replace("<>", "{}"));
            if (!singletonDirectoryInfo.Exists)
                singletonDirectoryInfo.Create();
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("{} - ", singletonDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (!string.IsNullOrEmpty(collectionDescription))
        {
            result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
            System.IO.DirectoryInfo collectionDirectoryInfo = new(result.Replace("<>", "[]"));
            if (!collectionDirectoryInfo.Exists)
                collectionDirectoryInfo.Create();
            checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("[] - ", collectionDescription));
            if (!Directory.Exists(checkDirectory))
                _ = Directory.CreateDirectory(checkDirectory);
        }
        if (string.IsNullOrEmpty(result))
            throw new Exception();
        results.Add(result);
        return results;
    }

}